import enum


class LocationType(enum.StrEnum):
    AUSTRALIA_NEW_ZEALAND = "Австралия и Новая Зеландия"
    ANTARCTICA = "Антарктика"
    EASTERN_ASIA = "Восточная Азия"
    EASTERN_AFRICA = "Восточная Африка"
    EASTERN_EUROPE = "Восточная Европа"
    TRANSCAUCASIA = "Закавказье"
    WESTERN_ASIA = "Западная Азия"
    WESTERN_AFRICA = "Западная Африка"
    WESTERN_EUROPE = "Западная Европа"
    INDIAN_OCEAN = "Индийский океан"
    CARIBBEAN = "Карибский бассейн"
    MELANESIA = "Меланезия"
    MICRONESIA = "Микронезия"
    POLYNESIA = "Полинезия"
    NORTHERN_AFRICA = "Северная Африка"
    NORTHERN_AMERICA = "Северная Америка"
    NORTHERN_EUROPE = "Северная Европа"
    CENTRAL_AFRICA = "Центральная Африка"
    CENTRAL_AMERICA = "Центральная Америка"
    SOUTHEAST_ASIA = "Юго-Восточная Азия"
    SOUTH_AMERICA = "Южная Америка"
    SOUTHERN_EUROPE = "Южная Европа"
    SOUTHERN_AFRICA = "Южная часть Африки"
    SOUTHERN_CENTRAL_ASIA = "Южная часть Центральной Азии"
    SOUTH_OCEAN = "Южный океан"
